package cz.mendelu.dnaAnalyser.analyse.g4hunter

import cz.mendelu.dnaAnalyser.utils.api.ApiRestTemplate
import cz.mendelu.dnaAnalyser.utils.controller.TagModifyRequest
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import org.springframework.test.context.ActiveProfiles
import spock.lang.Ignore
import spock.lang.Specification

import static org.springframework.boot.test.context.SpringBootTest.WebEnvironment.RANDOM_PORT

@Ignore
@SpringBootTest(webEnvironment = RANDOM_PORT)
@ActiveProfiles("test")
class G4hunterControllerTest extends Specification {

    @Autowired
    ApiRestTemplate api

    def setup() {
        api.authentication('user@mendelu.cz', 'user')
    }
    
    def "GET:/api/analyse/g4hunter Get page with G4hunter analysis."(){
        when:
        def response = api.get("/api/analyse/g4hunter")

        then:
        response.status == HttpStatus.OK.value()
    }

    def "GET:/api/analyse/g4hunter/{id} Get one G4hunter analyse by ID."(){
        setup:
        def id = UUID.fromString("b0cfb3e7-1e31-45a2-8dbc-0b31a2b39876")

        when:
        def response = api.get("/api/analyse/g4hunter/${id}")

        then:
        response.status == HttpStatus.OK.value()

    }

    def "POST:/api/analyse/g4hunter Create G4hunter analyse."() {
        given:
        def request = new G4HunterController.G4HunterCreateRequest(
                sequence: UUID.fromString('987acc67-5714-47b1-b56f-1dc56ded7c87'),
                threshold: 0.5,
                windowSize: 20
        )

        when:
        def response = api.post("/api/analyse/g4hunter", request)

        then:
        response.status == HttpStatus.CREATED.value()
    }

    def "DELETE:/api/analyse/g4hunter/{id} Delete G4hunter analyse by ID."(){
        setup:
        def id = UUID.fromString("1ec02567-17dc-4115-9d2f-dba9812b9a3f")

        when:
        def response = api.delete("/api/analyse/g4hunter/${id}")

        then:
        response.status == HttpStatus.NO_CONTENT.value()
    }

    @Ignore
    def "GET:/api/analyse/g4hunter/{id}/quadruplex/heatmap Get heatmap analyse by ID."(){
        setup:
        def id = UUID.fromString("b0cfb3e7-1e31-45a2-8dbc-0b31a2b39876")
        def segments = 80

        when:
        def response = api.get("/api/analyse/g4hunter/${id}/quadruplex/heatmap?segments=${segments}")

        then:
        response.status == HttpStatus.OK.value()
        response.body.data.size() == 75
    }

    @Ignore
    def "GET:/api/analyse/g4hunter/{id}/quadruplex Get quadruplex for analyse by ID."(){
        setup:
        def id = UUID.fromString("b0cfb3e7-1e31-45a2-8dbc-0b31a2b39876")
        def sequenceLength = 11
        def sequenceStart = 12 * sequenceLength

        when:
        def response = api.get("/api/analyse/g4hunter/${id}/quadruplex?sequenceStart=${sequenceStart}&sequenceLength=${sequenceLength}")

        then:
        response.status == HttpStatus.OK.value()
        response.body.items.size() == 4
    }

    def "PUT:/api/g4hunter/{id}/tags Modify tags"(){
        setup:
        def id = UUID.fromString('b0cfb3e7-1e31-45a2-8dbc-0b31a2b39876')
        def request = new TagModifyRequest(
                tags: ['new tags']
        )

        when:
        def response = api.put("/api/analyse/g4hunter/${id}/tags", request)

        then:
        response.status == HttpStatus.ACCEPTED.value()

        and:
        def payload = response.body.payload
        payload.tags == ['new tags']
    }

}
