package cz.mendelu.dnaAnalyser.analyse.palindrome.detector

import cz.mendelu.dnaAnalyser.analyse.palindrome.model.PalindromeModel
import cz.mendelu.dnaAnalyser.sequence.NucleicBuffer
import cz.mendelu.dnaAnalyser.sequence.data.SequenceData

/**
 * Created by xkoloma1 on 03.03.2016.
 */
class AtatPalindromeModelDetectorTest extends groovy.util.GroovyTestCase {

    static {
        SequenceTestSupport.stringAsSequence()
    }

    void "test IsAtat(): Valid"() {
        assert AtatPalindromeDetector.isAtat("ATATAT" as NucleicBuffer) == true
        assert AtatPalindromeDetector.isAtat("TATATA" as NucleicBuffer) == true
        assert AtatPalindromeDetector.isAtat("CGCGCG" as NucleicBuffer) == true
        assert AtatPalindromeDetector.isAtat("GCGCGC" as NucleicBuffer) == true
    }

    void "test IsAtat(): Invalid"() {
        assert AtatPalindromeDetector.isAtat("ATATTAT" as NucleicBuffer) == false
        assert AtatPalindromeDetector.isAtat("ATATCAT" as NucleicBuffer) == false
    }

    void "test findPalindrome in longer sequence"() {
        AtatPalindromeDetector detector = new AtatPalindromeDetector((4..6) as SortedSet, (2..3) as SortedSet)
        PalindromeMatcher matcher = detector.findPalindrome("CCATATATATATATATATATCC" as SequenceData)
        assert matcher.count == 1

        PalindromeModel palindrome = matcher.iterator().next()
        assert palindrome.sequence.toPlain() == "ATATAT"
        assert palindrome.opposite.toPlain() == "ATATAT"
        assert palindrome.spacer.toPlain() == "AT"
        assert palindrome.position == 2
    }

    void "test findPalindrome in longer sequence 2"() {
        AtatPalindromeDetector detector = new AtatPalindromeDetector((4..6) as SortedSet, (2..3) as SortedSet)
        PalindromeMatcher matcher = detector.findPalindrome("CCATATATATATATATATATACC" as SequenceData)
        assert matcher.count == 1

        PalindromeModel palindrome = matcher.iterator().next()
        assert palindrome.sequence.toPlain() == "ATATAT"
        assert palindrome.opposite.toPlain() == "ATATAT"
        assert palindrome.spacer.toPlain() == "AT"
        assert palindrome.position == 2
    }

    void "test findPalindrome in short sequence"() {
        AtatPalindromeDetector detector = new AtatPalindromeDetector((5..15) as SortedSet, (2..3) as SortedSet)
        PalindromeMatcher matcher = detector.findPalindrome("CCATATATATATATATATATATCC" as SequenceData)
        assert matcher.count == 1

        PalindromeModel palindrome = matcher.iterator().next()
        assert palindrome.sequence.toPlain() == "ATATATATAT"
        assert palindrome.opposite.toPlain() == "ATATATATAT"
        assert palindrome.spacer.length == 0
        assert palindrome.position == 2
    }

    void "test findPalindrome in short sequence 2"() {
        AtatPalindromeDetector detector = new AtatPalindromeDetector((5..15) as SortedSet, (2..3) as SortedSet)
        PalindromeMatcher matcher = detector.findPalindrome("CCATATATATATATATATATATACC" as SequenceData)
        assert matcher.count == 1

        PalindromeModel palindrome = matcher.iterator().next()
        assert palindrome.sequence.toPlain() == "ATATATATAT"
        assert palindrome.opposite.toPlain() == "ATATATATAT"
        assert palindrome.spacer.length == 0
        assert palindrome.position == 2
    }

    void "test findPalindrome in middle sequence"() {
        AtatPalindromeDetector detector = new AtatPalindromeDetector((5..7) as SortedSet, (3..7) as SortedSet)
        PalindromeMatcher matcher = detector.findPalindrome("CCATATATATATATATATATCC" as SequenceData)
        assert matcher.count == 1

        PalindromeModel palindrome = matcher.iterator().next()
        assert palindrome.sequence.toPlain() == "ATATATA"
        assert palindrome.opposite.toPlain() == "TATATAT"
        assert palindrome.spacer.toPlain() == "TATA"
        assert palindrome.position == 2
    }

    void "test findPalindrome in middle sequence 2"() {
        AtatPalindromeDetector detector = new AtatPalindromeDetector((5..7) as SortedSet, (3..7) as SortedSet)
        PalindromeMatcher matcher = detector.findPalindrome("CCATATATATATATATATATACC" as SequenceData)
        assert matcher.count == 1

        PalindromeModel palindrome = matcher.iterator().next()
        assert palindrome.sequence.toPlain() == "ATATATA"
        assert palindrome.opposite.toPlain() == "TATATAT"
        assert palindrome.spacer.toPlain() == "TATA"
        assert palindrome.position == 2
    }
}
