package cz.mendelu.dnaAnalyser.utils.analyse.data;

import org.apache.ibatis.annotations.*;
import org.apache.ibatis.session.RowBounds;

import java.util.List;

public interface TestDataMapper extends DataLocalisedMapper<TestData> {

    // language=SQL
    String TABLE = "CREATE TABLE TEST_DATA (" +
            "ID integer NOT NULL AUTO_INCREMENT, " +
            "POSITION integer, " +
            "LENGTH integer, " +
            "VALUE varchar(255), " +
            "PRIMARY KEY (ID)" +
            ")";

    // language=SQL
    @Select("SELECT count(*) FROM TEST_DATA")
    int countAll();

    // language=SQL
    @Select("SELECT * FROM TEST_DATA")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "value", column = "VALUE")
    })
    List<TestData> getAll(RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM TEST_DATA WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} ASC")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "value", column = "VALUE")
    })
    List<TestData> getAllBetweenPositionOrderAsc(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition,
            @Param("orderName") String orderName,
            RowBounds rowBounds);

    // language=SQL
    @Select("SELECT * FROM TEST_DATA WHERE ID = #{id}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "POSITION"),
            @Result(property = "length", column = "LENGTH"),
            @Result(property = "value", column = "VALUE")
    })
    TestData getById(int id);

    // language=SQL
    @Update("UPDATE TEST_DATA SET POSITION = #{position}, LENGTH = #{length}, VALUE = #{value} WHERE ID = #{id}")
    void update(TestData data);

    // language=SQL
    @Delete("DELETE from TEST_DATA WHERE ID = #{id}")
    void delete(int id);

    // language=SQL
    @Insert("INSERT INTO QUADRUPLEX (ID, POSITION, LENGTH, VALUE) VALUES (#{id}, #{position}, #{length}, #{value})")
    @Options(useGeneratedKeys = true, keyProperty = "id")
    void insert(TestData data);

}